import React from 'react';
import { connect } from 'react-redux';
import mqtt from 'mqtt';
import Sensor from "./sensor";

import Affichage from "./Affichage";
import Menu from "./Menu";

import logo from './logo.svg';

import './App.css';




const prefix = "value/";

let id = 0;
let routes = [];

const connectFormAction = (url) => (dispatch,getState) => {

	return new Promise((accept, reject) => {

		//Connexion par défaut
		if(url === "") url = "ws://127.0.0.1:1234";

		let socket = mqtt.connect(url);

		socket.on("message", function(topic, payload) {

			var name = topic.replace(prefix,"");

			var tmp = [topic, payload].join("|").split('|')[1];
			var tmp2 = JSON.parse(tmp);

			var dt = new Date();
			var date = dt.toISOString();

			var index = -1;

			var states = getState().sensorReducer;

			for(let i=0; i<states.length; i++){
				if(states[i].sensor.name === name){
					index = i;
					break;
				}
			}

			if (index === -1) {		

				var sensor = new Sensor(name, tmp2.value, tmp2.type);

				dispatch({
					type:"ADD_SENSOR",
					id:id,
					sensor:sensor
				});

				id++;

				//MENU
				var nameURL = encodeURI(sensor.name);

				if(!routes.includes(sensor.name)) routes.push(sensor.name);

				dispatch({
					type:'ADD_MENU',
					name:name,
					url:nameURL
				});

			}
			else {
				dispatch({
					type:"ADD_VALUE",
					id:index,
					value:tmp2.value,
					label:date
				});

			}

		});

		socket.on('connect', () => {
			socket.subscribe(prefix + '#');
			accept();
		});

	}).then(() => dispatch({type:'REQUEST_CONNECTION',value:false}))


}

class App extends React.Component {

	render() {

		let input;

		return(

			<div>
				<header className="App-header">
					<img src={logo} className="App-logo" alt="logo" />
					<h1 className="App-title">Lab Redux</h1>
				

					<div className="haut">

						<div className="formulaire">
							<form onSubmit={e => {
								e.preventDefault();
								e.stopPropagation();

								this.props.dispatch(connectFormAction(input.value));
								input.value = '';
							}}>
								<input ref={node => {input = node}} />
								<button type="submit">Connexion</button>
							</form>
						</div>
					</div>

				</header>

				<div className="wrapper">
					<Menu />
					<Affichage url={this.props.location.pathname}/>
				</div>
			</div>
		
		)
	}
}


App = connect(
	function mapStateToProps(state) {
		return {
			sensor: state.sensorReducer,
			menu: state.menuReducer
		}
	}
)(App);

export default App;
